#include <Adafruit_SleepyDog.h>

// The Tone() function uses a huge amount of memory so the code does not fit into an ATtiny25
// Uncomment this line to use the Tone() function on an ATtiny45/85
// #define USE_TONES

#define BUZZER_PIN 0
#define PIN_BUTTON 1

#define SLEEP_PERIOD 1000U
#define SECONDS_IN_MINUTE 60U

#define WORK_CYCLES 4

#define WORK_MINUTES 25
#define SHORT_PAUSE_MINUTES 5
#define LONG_PAUSE_MINUTES 20

#define REST_CYCLES WORK_CYCLES - 1

#define WORK_DURATION WORK_MINUTES * SECONDS_IN_MINUTE
#define SHORT_REST_DURATION SHORT_PAUSE_MINUTES * SECONDS_IN_MINUTE
#define LONG_REST_DURATION LONG_PAUSE_MINUTES * SECONDS_IN_MINUTE

#ifdef USE_TONES
const uint8_t tone_sequence[] = {130, 155, 196, 233, 130, 155, 196, 233};
#endif

typedef enum state { working, short_pause, long_pause } state;

state current_state;
uint8_t current_cycle;
uint16_t elapsed_time;

void setup() {
#ifdef ADCSRA
  ADCSRA = 0; // Disable ADC hardware
#endif
#ifdef ACSR
  ACSR = 0x80; // Disable analog comparator
#endif
  PRR = 0xFF; // Reduce power consumption
  pinMode(BUZZER_PIN, OUTPUT);
  digitalWrite(BUZZER_PIN, LOW);
  pinMode(PIN_BUTTON, INPUT_PULLUP);
  current_cycle = 0;
  elapsed_time = 0;
  current_state = working;
}

void loop() {
  Watchdog.sleep(SLEEP_PERIOD);
  elapsed_time++;
  switch (current_state) {
  case working:
    if (elapsed_time >= WORK_DURATION) {
      elapsed_time = 0;
      if (++current_cycle < WORK_CYCLES) {
        restBuzz(false);
        current_state = short_pause;
      } else {
        current_cycle = 0;
        restBuzz(true);
        current_state = long_pause;
      }
    }
    break;

  case short_pause:
    if (elapsed_time >= SHORT_REST_DURATION) {
      elapsed_time = 0;
      workBuzz();
      current_state = working;
    }
    break;

  case long_pause:
    if (elapsed_time >= LONG_REST_DURATION) {
      elapsed_time = 0;
      workBuzz();
      current_state = working;
    }
    break;

  default:
    break;
  }
  // Testing button
  if (digitalRead(PIN_BUTTON) == LOW) {
    if (debounce(PIN_BUTTON)) {
      tellCurrentState();
    }
  }
}

void tellCurrentState(void) {
  switch (current_state) {
  case long_pause:
    digitalWrite(BUZZER_PIN, HIGH);
    Watchdog.sleep(50);
    digitalWrite(BUZZER_PIN, LOW);
    Watchdog.sleep(50);
  case short_pause:
    digitalWrite(BUZZER_PIN, HIGH);
    Watchdog.sleep(50);
    digitalWrite(BUZZER_PIN, LOW);
    Watchdog.sleep(50);
  case working:
    digitalWrite(BUZZER_PIN, HIGH);
    Watchdog.sleep(50);
    digitalWrite(BUZZER_PIN, LOW);
    break;

  default:
    digitalWrite(BUZZER_PIN, LOW);
    break;
  }
}

void restBuzz(bool long_rest) {
#ifdef USE_TONES
  uint8_t index;
  if (long_rest) {
    index = 0;
  } else {
    index = 4;
  }
  for (; index < 8; index++) {
    tone(BUZZER_PIN, tone_sequence[index]);
    delay(250);
  }
  noTone(BUZZER_PIN);
#else
  if (long_rest) {
    for (uint8_t i = 0; i < 4; i++) {
      digitalWrite(BUZZER_PIN, HIGH);
      Watchdog.sleep(250);
      digitalWrite(BUZZER_PIN, LOW);
      Watchdog.sleep(50);
    }
  } else {
    digitalWrite(BUZZER_PIN, HIGH);
    Watchdog.sleep(100);
    digitalWrite(BUZZER_PIN, LOW);
    Watchdog.sleep(50);
    digitalWrite(BUZZER_PIN, HIGH);
    Watchdog.sleep(100);
    digitalWrite(BUZZER_PIN, LOW);
  }
#endif
}

void workBuzz(void) {
#ifdef USE_TONES
  for (uint8_t i = 7; i > 3; i--) {
    tone(BUZZER_PIN, tone_sequence[i]);
    delay(250);
  }
  noTone(BUZZER_PIN);
#else
  for (uint8_t i = 0; i < 8; i++) {
    digitalWrite(BUZZER_PIN, HIGH);
    Watchdog.sleep(100);
    digitalWrite(BUZZER_PIN, LOW);
    Watchdog.sleep(100);
  }
#endif
}

bool debounce(uint8_t pin) {
  uint8_t initialReading = digitalRead(pin);
  Watchdog.sleep(100);
  return (initialReading == digitalRead(pin));
}
